@@ -25,6 +25,10 @@ struct qhess { REAL **gg;};
 typedef struct method_instance *MIPTR;
 #endif
 
+extern int gen_quant_list_max;
+extern int gen_quant_free_left;
+extern int meth_inst_list_max;
+
 #define MAXVCOUNT 100
 /* structure for passing info to element methods */
 struct qinfo {
@@ -110,6 +114,7 @@ struct gen_quant {
 #define Q_REDUNDANT     0x10000  /* skip this quantity due to torus_filled */
 #define TORUS_MODULO_MUNGE 0x40000
 #define Q_DELETED  0x80000
+#define Q_FORWARD_DEF  0x100000
 
 /* structure for one quantity calculating method */
 /* split so can use same method for many quantities, */
@@ -161,6 +166,9 @@ struct method_instance {
   int flags;
   int gen_method;  /* parent method */
   int quant;  /* quantity this is a part of */
+  int connum; /* constraint or bdry this may be associated with */
+  int quant_index; /* order of method in quantity list */
+  int global_low_rank;   /* index in list of all instances used in low rank updates */
   int vec_order;  /* dimension of k-vector */
   int elmodulus;  /* number of extra attribute to use as element modulus */
   struct expnode *expr[MAXMEXPR];  /* whatever interpreted expressions needed */
@@ -193,7 +201,6 @@ struct method_instance {
 /*  struct method_instance *next; */  /* linked list */
   };
 /* flags, (avoid quantity flag bits) */
-#define GLOBAL_INST  0x100000
 #define IMPLICIT_INSTANCE 0x200000
 #define DEFAULT_INSTANCE 0x400000
 #define METH_PARAMETER_1 0x800000
@@ -203,6 +210,7 @@ struct method_instance {
 #define IGNORE_FIXED    0x8000000
 #define ELEMENT_MODULUS_FLAG 0x10000000
 #define USE_DENSITY     0x20000000
+#define GLOBAL_INST     0x40000000
 /* start for self_id, to distinguish from bodies and quantities */
 #define METHBASE 0x10000
 
@@ -305,6 +313,14 @@ extern REAL spherical_arc_area_s_value(QINFO);
 extern REAL spherical_arc_area_s_grad(QINFO);
 extern REAL spherical_arc_area_s_hess(QINFO);
 
+extern REAL null_length_value(QINFO);
+extern REAL null_length_grad(QINFO);
+extern REAL null_length_hess(QINFO);
+
+extern REAL null_area_value(QINFO);
+extern REAL null_area_grad(QINFO);
+extern REAL null_area_hess(QINFO);
+
 
 extern REAL q_edge_area(QINFO);
 extern REAL q_edge_area_grad(QINFO);
@@ -438,12 +454,20 @@ extern REAL sqcurve_string_value(QINFO);
 extern REAL sqcurve_string_grad(QINFO);
 extern REAL sqcurve_string_hess(QINFO);
 extern void sqcurve_string_marked_init ARGS((QINT,struct method_instance*));
+extern REAL curve_power;
+extern int curve_power_param;
+#define CURVE_POWER_NAME "curvature_power"
 
 extern void sqcurve2_string_init ARGS((QINT,struct method_instance*));
 extern REAL sqcurve2_string_value(QINFO);
 extern REAL sqcurve2_string_grad(QINFO);
 extern REAL sqcurve2_string_hess(QINFO);
 
+extern void sqcurve3_string_init ARGS((QINT,struct method_instance*));
+extern REAL sqcurve3_string_value(QINFO);
+extern REAL sqcurve3_string_grad(QINFO);
+extern REAL sqcurve3_string_hess(QINFO);
+
 extern void sq_mean_curv_cyl_init ARGS((QINT,struct method_instance*));
 extern REAL sq_mean_curv_cyl_value(QINFO);
 extern REAL sq_mean_curv_cyl_grad(QINFO);
@@ -581,6 +605,10 @@ extern REAL star_sqcurve_method_value(QINFO);
 extern REAL star_sqcurve_method_grad(QINFO);
 extern REAL star_sqcurve_method_hess(QINFO);
 
+extern void laplacian_mean_curvature_init ARGS((QINT,struct method_instance*));
+extern REAL laplacian_mean_curvature_value(QINFO);
+extern REAL laplacian_mean_curvature_grad(QINFO);
+
 extern void stokes2d_init ARGS((QINT,struct method_instance*));
 extern REAL stokes2d_value(QINFO);
 extern REAL stokes2d_grad(QINFO);
@@ -645,6 +673,12 @@ extern REAL relaxed_elastic2_A_energy(QINFO);
 extern REAL relaxed_elastic2_A_gradient(QINFO);
 extern REAL relaxed_elastic2_A_hessian(QINFO);
 
+extern void dirichlet_elastic_init ARGS((QINT,struct method_instance*));
+extern REAL dirichlet_elastic_energy(QINFO);
+extern REAL dirichlet_elastic_gradient(QINFO);
+extern REAL dirichlet_elastic_hessian(QINFO);
+
+
 extern void SVK_init ARGS((QINT,struct method_instance*));
 extern REAL SVK_energy(QINFO);
 extern REAL SVK_gradient(QINFO);
@@ -702,6 +736,10 @@ extern void facet_knot_energy_init ARGS((QINT,struct method_instance*));
 extern REAL facet_knot_energy(QINFO);
 extern REAL facet_knot_energy_gradient(QINFO);
 
+extern void bi_surface_init ARGS((QINT,struct method_instance*));
+extern REAL bi_surface_energy(QINFO);
+extern REAL bi_surface_gradient(QINFO);
+
 extern void facet_knot_energy_fix_init ARGS((QINT,struct method_instance*));
 extern REAL facet_knot_energy_fix(QINFO);
 extern REAL facet_knot_energy_fix_gradient(QINFO);
